/* i2csubs.c
 * Copyright (C) 2013-2014 Jim Jackson  <jj@franjam.org.uk>
 *
 * Linux I2C - initially on RaspberryPI 
 * i2c subs
 */
/*
 *  Date       Vers   Comment
 *  -----      -----  ---------------
 *  2013-01-07  1     i2c subs moved out to separate file
 *  2013-09-16  2     changes made for possible use of ioctl versions
 *                    of i2c_read and i2c_write
 */

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program - see the file COPYING; if not, write to 
 *  the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
 *  MA 02139, USA.
 */

/* INCLUDES .................................................... */

#include <stdlib.h>  
#include <stdio.h>   /* Standard input/output definitions */
#include <string.h>  /* String function definitions */
#include <unistd.h>  /* UNIX standard function definitions */
#include <fcntl.h>   /* File control definitions */
#include <errno.h>   /* Error number definitions */
#include <ctype.h>
/* #include <linux/i2c.h> */
#include <linux/i2c-dev.h>
#include "i2csubs.h"

/* i2cinit(dev,add)  open i2c device dev and, if add > 0 , set the i2c
 *  device address to add.
 *    return -1 on error with errno set, else return file descriptor
 */

int i2c_init(char *dev,int add) {
  int fd,st;

  fd = open(dev, O_RDWR);
  if (fd == -1) { return(-1); }
  if (i2c_setadd(fd,add)==0) { return(fd); }
  st=errno;
  close(fd);
  errno=st;
  return(-1);
}

/* i2c_setadd(fd,add)  set add as i2c address on i2c device fd
 *    return 0 if all ok else return -1 with errno set to error
 */

int i2c_setadd(int fd, int add) {
  if ((add>0) && ioctl(fd,I2C_SLAVE,add)) { return(-1); }
  return(0);
}

/* i2c_close(fd)   a wrapper for the close() function
 */

int i2c_close(int fd) {
  return(close(fd));
}

/* i2c_getreg(fd,reg,val)  read an 8 bit register, reg, on an i2c device
 *   return 0 if ok and *val is the returned value, else an error number
 *      is returned.
 */

int i2c_getreg(int fd, int reg, int *val) {
  unsigned char bf[4];
  int st;

  *bf=reg;
  if ( (st=i2c_write(fd,bf,1)) != 1 ) {
    return((st==-1)?errno:EIO);
  }
  if ( (st=i2c_read(fd,bf,1)) != 1 ) {
    return((st==-1)?errno:EIO);
  }
  *val=bf[0];
  return(0);
}

/* i2c_setreg(fd,reg,val)  set an 8 bit register, reg, on an i2c device
 *   to val
 *   return 0 if ok else an error number is returned.
 */

int i2c_setreg(int fd, int reg, int val) {
  unsigned char bf[4];
  int st;
  bf[0]=reg; bf[1]=val;
  
  if ( (st=i2c_write(fd,bf,2)) != 2 ) {
    return((st==-1)?errno:EIO);
  }
  return(0);
}

