/* lcddriver.h   version for jj i2c mcp23017 based connection to LCD
 * prototypes and constants etc for the functions in lcddriver.c
 * Jim Jackson <jj@franjam.org.uk> 
 *
 */

#ifndef __LCDDRIVER_H
#define __LCDDRIVER_H

#ifndef FALSE
# define FALSE (0)
# define TRUE (1)
#endif

#define INTERFACE_NAME "jj.i2c"

/* lcd_wait(p)  pauses for p millisecs or the timersubs tick, whichever
 *                is the bigger
 */

void lcd_wait(unsigned int p) ;

/* lcd_reinit()  calls lcd_init with null args so the LCD is simply
 *               re-initialised
 */

#define lcd_reinit()  lcd_init(NULL,0,0,0)   /* call to init to reset lcd */

/* lcd_put(c)   writes the byte c to the lcd as data
 * lcd_cmd(c)   writes the byte c to the lcd as a HD44780 cmd
 */

int lcd_put(int c);
int lcd_cmd(int c);

#define DEFAULT_CURSORMODE (2) // set to default cursor

/* lcd_bel(period,type)  performs the action of a BELL of type type
 *                       could be a sounder, or a visual bell e.g. flashing
 *                       the lcd backlight. period is number of millisecs
 *                       for the action.
 */

int lcd_bel(int period, int type);

/* defines for the lcd_bel type. These are bit values and can be or'ed for
 * multiple actions - e.g. visual and sounder.
 */

#define DEFAULT_BELL (1)     // what this driver considers a default
#define AUDIO_BELL (2)   //
#define VISUAL_BELL (1)  // 
#define REPEAT_BELL (256) // to a repetitive version of type of bell

/* int lcd_backlight(int value)  driver dependant. Use value to set backlight.
 *                               often just 0=off, non-0=on
 */

int lcd_backlight(int value);

#define DEFAULT_BACKLIGHT (1)  /* edit this to the default value for the 
				  backlight value */

#endif  /* __LCDDRIVER_H */
