/* timersubs.h   
 * prototypes and constants etc for the functions in timersubs.c
 * Jim Jackson <jj@franjam.org.uk> 
 *
 */

#ifndef __TIMERSUBS_H
#define __TIMERSUBS_H

/* Size of internal counter array. This defines the number of counters 
 * that can configured. Don't make it too big as the timer_handler code
 * has to check each entry every tick.
 */

#define MAX_COUNTERS (8)

/* Macro that waits for a counter to get to zero ...
 */

#define waitcounter(C) while ( (C) != 0 ) { waitalarm(); }

/* init_timers()  called to setup the timer functions... 
 *  unsigned int ms   number of millisecs between timer_handler calls
 *                    determines the resolutions of the timers
 *                    e.g. 10 millisecs will set the timer_handler
 *                    running 100 times a second. 
 *                    The timer_handler will decrement each millisec 
 *                    counter by 10 each call.
 */

int init_timers(unsigned int ms);

/* waitalarm()   suspends until the timer_handler is next invoked
 *               when waiting for a counter to become zero, calling this
 *               between checks provides a more efficient way of waiting than
 *               simply looping of the test.
 *
 *                while (counter) { waitalarm(); }
 */

void waitalarm();

/* counter_setup()  called to setup a counter
 *  unsigned int *cp  pointer to callers counter
 *                    when this variable is non-zero it will be decremented
 *                    every millisecond until it is zero
 *                    The counter can be set by the calling code or
 *                    by this code if a non-zero ms value is given
 *  unsigned int ms   number of milliseconds to initialise the counter 
 *                    with now and each time the counter decrements to zero
 *  void (*func)()    pointer to a function to call when the counter is 
 *                    decremented to zero, or NULL
 */

int counter_setup(unsigned int *cp, unsigned int ms, void (*func)());

/* counter_remove()  called to remove an instance of a counter
 *  unsigned int *cp  pointer to find and remove entry for.
 */

int counter_remove(unsigned int *cp);

#endif  /* __TIMERSUBS.H */
