/* timtest.c 
 * Test program for the timersubs.c code
 *
 * Jim Jackson
 */
/*
 * Copyright (C) 2005-2014 Jim Jackson           jj@franjam.org.uk
 */
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program - see the file COPYING; if not, write to 
 *  the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
 *  MA 02139, USA.
 */
/*
 *  Date     Vers    Comment
 *  -----    -----   ---------------
 *   4Jan14  0.1     
 */

/***INCLUDES***/
/* includes files 
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
/* for SYSV variants .... */
#include <string.h>
/* #include <strings.h> */
#include <errno.h>
#include <fcntl.h>
#include <ctype.h>
#include <math.h>
#include <time.h>
#include <sys/time.h>
#include "timersubs.h"

/***DEFINES***/
/* defines
 */

#define VERSION "0.1 beta"

#define PVERSION(A) fprintf(stderr,"%s version %s\n",sys,A)

#define EFAIL -1

#define TRUE 1
#define FALSE 0
#define nl puts("")

#define chkword(a,b) ((n>=a)&&(strncmp(p,b,n)==0))
#define index(A,B) strchr(A,B)
#define rindex(A,B) strrchr(A,B)

/* default timeformat for -T option - see man strftime */
#define TIMEFORMAT "%Y-%m-%d:%T "

/***GLOBALS***/
/* global variables
 */

char *sys;
int vflg;                        /* set if -v flag used        */
int timestamp;                   /* -t add timestamp to each line */
char *timeformat;

int mspause;
int counter1;
int counter2;

struct timeval start,tm,tmlast;
struct timezone tz;

/***DECLARATIONS***/
/* declare non-int functions 
 */

char *delnl(),*strupper(),*reads(),*strrchr(),*sindex(),
  *sindex_nc(),*tab(),*ctime(),*getcmdline();

int dotimestamp(FILE *f) {
  char bf[130];
  unsigned int t;

  if ( gettimeofday(&tm, &tz) ) {
    exit(err_rpt(errno,"Getting time of day."));
  }
  if (timestamp==1) {
    fprintf(f,"%lu.%03ld ",tm.tv_sec,(tm.tv_usec+500)/1000);
  } else {
    strftime(bf,128,timeformat,localtime(&tm.tv_sec));
    fputs(bf,f);             
  }
}

/* periodicoutput()
 */

void periodicoutput() {
  static int j=0;

  j++;
  if ( gettimeofday(&tm, &tz) ) {
    exit(err_rpt(errno,"Getting time of day."));
  }
  printf("%lu.%03ld %d\n",tm.tv_sec,(tm.tv_usec+500)/1000,j);
}


/***MAIN***/
/* main - for program  lcdd
 */

main(argc,argv)
int argc;
char **argv;

{ int i,j,n,t,st;
  FILE *f; 
  char *p;
  char buf[130];

  argv[argc]=NULL;
  sys=*argv++; argc--;
  if ((p=rindex(sys,'/'))!=NULL) { sys=p+1; }

/* now do option setup and parsing... */
  vflg=FALSE;
  timestamp=1;
  timeformat=TIMEFORMAT;
  mspause=1000;

  /* put global and local var. init. here */

  while (argc && **argv=='-') {          /* all flags and options must come */
    n=strlen(p=(*argv++)+1); argc--;     /* before parameters                */
    if (chkword(1,"pause")) {             /* -size option              */
      if (argc && (**argv!='-') && isdigit(**argv)) { 
	mspause=atoi(*argv++); argc--;
      }
    } else {                             /* check for single char. flags    */
      for (; *p; p++) {
	if (*p=='h') exit(help(EFAIL));
	else if (*p=='v') vflg=TRUE;
	else if (*p=='t') timestamp=1;
	else if (*p=='T') timestamp=2;
	else {
          *buf='-'; *(buf+1)=*p; *(buf+2)=0;
          exit(help(err_rpt(EINVAL,buf)));
	}
      }
    }
  }
  
  if ( argc>0 ) {
    exit(err_rpt(EINVAL,"Too many command line parameters."));
  }


  init_timers(mspause);
  dotimestamp(stdout);
  if (st=counter_setup(&counter1,2000,&periodicoutput)) {
    exit(err_rpt(st,"setting up counter1."));
  }
  puts(" set counter1 repeating every 2000");

  if (st=counter_setup(&counter2,0,NULL)) {
    exit(err_rpt(st,"setting up counter1."));
  }

  dotimestamp(stdout);
  counter2=5000;
  puts(" set counter2 to do one off 2000");

  for ( ; ; ) {
    if (counter2==0) {
      dotimestamp(stdout);
      printf("Counter2 finished.\n");
      dotimestamp(stdout);
      counter2=6000;
      printf("Counter2 set to 6000.\n");

    }
    waitalarm();
  }

  exit(0);
}

/***HELP***/
/* help(e)  print brief help message - return parameter given
 */

help(e)
int e;
{
  PVERSION(VERSION);
  puts("\n\
timtest - testbed for timersubs.c\n\
");
  printf("\
Usage:   %s [-v] [-t] [-p pause] \n\n\
",sys);
  printf("\
  pause  values are in millisecs - e.g. 500 for half a sec\n\n");
  return(e);
}

/***ERR_RPT***/
/* err_rpt(err,msg)
 */

err_rpt(err,msg)
short int err;
char *msg;

{ 
  if (err) fprintf(stderr,"[%s] (%d) %s : %s\n",sys,err,strerror(err),msg);
  return(err);
}

